@@ -14,7 +14,7 @@
 #
 #    You should have received a copy of the GNU General Public License
 #    along with this program; if not, write to the Free Software
-#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
 import sys, time
 from UIBase import UIBase
@@ -33,16 +33,19 @@ class Basic(UIBase):
             warntxt = 'warning'
         sys.stderr.write(warntxt + ": " + str(msg) + "\n")
 
-    def sleep(s, sleepsecs):
+    def sleep(s, sleepsecs, siglistener):
         if s.verbose >= 0:
             s._msg("Sleeping for %d:%02d" % (sleepsecs / 60, sleepsecs % 60))
-        UIBase.sleep(s, sleepsecs)
+        return UIBase.sleep(s, sleepsecs, siglistener)
 
     def sleeping(s, sleepsecs, remainingsecs):
         if sleepsecs > 0:
             time.sleep(sleepsecs)
         return 0
 
+    def locked(s):
+        s.warn("Another OfflineIMAP is running with the same metadatadir; exiting.")
+
 class Quiet(Basic):
     def __init__(s, config, verbose = -1):
         Basic.__init__(s, config, verbose)
