@@ -23,13 +23,6 @@ try:
 except ImportError:
     pass
 
-try:
-    import Tkinter
-except ImportError:
-    pass
-else:
-    import Tk
-
 try:
     import curses
 except ImportError:
@@ -38,6 +31,7 @@ else:
     import Curses
 
 import Noninteractive
+import Machine
 
 # Must be last
 import detector
